#include "Game.h"

#define MAX_LEVEL 15

static int lives = BASE_LIVES;
static int actualLevel = BASE_LEVEL;
static int delay = BASE_DELAY;
static int poweredLED[MAX_LEVEL];
static int passedTries = 0;


void init_simon_game() {
	if(actualLevel > MAX_LEVEL) {
		on_game_end();
	}

	for(int i = 0; i<actualLevel; i++) {
		int random = rand() % LED_COUNT;

		poweredLED[i] = random;
	}
}

void show_game_next() {
	if(ledState == WAIT_STATE) {
		nextLedState++;

		if(nextLedState >= NEXT_STATE_DELAY) {
			switch_led_off(poweredLED[nextLed]);
			ledState = IDLE_STATE;
			nextLedState = 0;
			nextLed++;
			_delay_ms(50);
		}
		return;
	}

	if(ledState == GAME_STATE) {
		execute_simon_game();
		return;
	}

	if(ledState == IDLE_STATE) {
		if(nextLed >= actualLevel) {
			ledState = GAME_STATE;
			nextLed = 0;
			return;
		}

		switch_led_on(poweredLED[nextLed]);
		ledState = WAIT_STATE;	
	}
}



int get_button_pressed() {
	if(!(PIND & (1 << 4))) {
		return 0;
	}
	if(!(PIND & (1 << 6))) {
		return 1;
	}
	if(!(PINB & (1 << 1))) {
		return 2;
	}
	
	if(!(PINB & (1 << 4))) {
		return 99;
	}

	return -1;
}

void execute_simon_game() {
	int ledNumber = get_button_pressed();

	if(ledNumber == 99) {
		for(uint8_t i = 0; i<lives; i++) {
			switch_led_on(i);
		}
		return;
	} else {
		for(uint8_t i = 0; i<LED_COUNT; i++) {
			switch_led_off(i);
		}
	}

	if(ledNumber < 0) return;

	if(poweredLED[passedTries] == ledNumber) {
		on_pass(ledNumber);
	} else {
		on_lose();
	}
}

void on_pass(uint8_t ledNumber) {
	switch_led_on(ledNumber);
	_delay_ms(delay);

	switch_led_off(ledNumber);
	_delay_ms(delay);
	passedTries++;

	if(passedTries >= actualLevel) {
		on_win();
	}
}

void on_lose() {
	lives--;
	_delay_ms(BASE_DELAY);
	if(lives <= 0) {
		signal_lose();
		actualLevel = BASE_LEVEL;
		lives = BASE_LIVES;
		delay = BASE_DELAY;
		reset_simon_game();
	}
}

void on_win() {
	signal_win();
	actualLevel++;
	delay = BASE_DELAY - actualLevel > 8 ? (actualLevel * 10) : 80;
	reset_simon_game();
}

void signal_win() {
	for(int j = 0; j<5; j++) {
		for(int i = 0; i<LED_COUNT; i++) {
			switch_led_on(i);
			_delay_ms(LED_WIN_LOSE_DELAY);
			switch_led_off(i);
			_delay_ms(LED_WIN_LOSE_DELAY);
		}
	}
}

void signal_lose() {
	for(int j = 0; j<5; j++) {
		switch_led_on(0);
		_delay_ms(LED_WIN_LOSE_DELAY);
		switch_led_off(0);
		_delay_ms(LED_WIN_LOSE_DELAY);
	}
}


void on_game_end() {
	for(;;) {
		for(int i = 0; i<LED_COUNT; i++) {
			switch_led_on(i);
			_delay_ms(LED_WIN_LOSE_DELAY);
			switch_led_off(i);
			_delay_ms(LED_WIN_LOSE_DELAY);
		}
	}
}

void reset_leds() {
	for(int i = 0; i<LED_COUNT; i++) {
		switch_led_off(i);
	}
}


void reset_simon_game() {
	reset_leds();
	
	passedTries = 0;
	ledState = IDLE_STATE;
	init_simon_game();
}

void switch_led_on(uint8_t ledNumber) {
	PORTD |= (1 << ledNumber);
}

void switch_led_off(uint8_t ledNumber) {
	PORTD &= ~(1 << ledNumber);
}

